@@ -1385,10 +1385,10 @@ int indicatorp(int addr)
 {
     if (structurep(addr) && length(addr) == 3 && eqlp(car(addr), SLASH) &&
 	(atomp(cadr(addr)) || variablep(cadr(addr))) &&
-	(atomp(caddr(addr)) || variablep(caddr(addr))))
+	(integerp(caddr(addr)) || variablep(caddr(addr))))
 	return (1);
     else
-	return (NO);
+	return (0);
 }
 
 
@@ -1559,7 +1559,7 @@ int remove_duplicate(int x)
 {
     if (nullp(x))
 	return (NIL);
-    else if (memq(car(x), cdr(x)))
+    else if (memberp(car(x), cdr(x)))
 	return (remove_duplicate(cdr(x)));
     else
 	return (listcons(car(x), remove_duplicate(cdr(x))));
@@ -1864,13 +1864,14 @@ int unify(int x, int y, int th)
 
 //typed unify. y is a pair list e.g. [L|Ls]
 int unify_pair(int x, int y, int th)
-{
+{	
     if (variablep(x)) {
+		//bindsym(x, y, th);
 	if (alpha_variable_p(x))
 	    variant[x - CELLSIZE][th] = y;
 	else if (atom_variable_p(x))
 	    SET_CAR(x, y);
-	push_stack(x, th);
+		push_stack(x, th);
 	return (YES);
     } else if (!listp(x))
 	return (NO);
@@ -2273,8 +2274,6 @@ int structure_to_list(int x)
 	res = listcons(car(x), res);
 	x = cdr(x);
     }
-    if (operatorp(car(res)))
-	res = listcons(makeconst(GET_NAME(car(res))), cdr(res));
 
     return (res);
 }
@@ -2335,6 +2334,15 @@ int add_prefix(int x)
 	return (add_prefix1(x));
 }
 
+
+int copy_pred(int x)
+{
+    if (structurep(x))
+	return (cons(car(x), copy_heap(cdr(x))));
+    else
+	return (x);
+}
+
 int exportp(int x)
 {
     int i, name, arity;
@@ -2345,8 +2353,7 @@ int exportp(int x)
     } else {
 	name = car(x);
 	arity = length(cdr(x));
-    }
-
+    } 
 
     for (i = 0; i < export_pt; i++) {
 	if (eqlp(export_data[i][0], name) && export_data[i][1] == arity) {
@@ -2363,10 +2370,12 @@ int copy_heap(int x)
 	return (NIL);
     else if (IS_ALPHA(x))
 	return (alpha_to_variable(x));
+	else if (operationp(x) && car(x) == SLASH)
+	return (x);
     else if (predicatep(x) && car(x) == NCURL)
 	return (x);
     else if (module_flag && predicatep(x) && exportp(x))
-	return (x);
+	return (copy_pred(x));
     else if (module_flag && predicatep(x) && !exportp(x))
 	return (add_prefix(x));
     else if (singlep(x))
@@ -2379,6 +2388,8 @@ int copy_heap(int x)
 	return (listcons(copy_heap(car(x)), copy_heap(cdr(x))));
     else if (car(x) == NECK || car(x) == AND || car(x) == OR)
 	return (list3(car(x), copy_heap(cadr(x)), copy_heap(caddr(x))));
+    else if (user_operation_p(x))
+	return (x);
     return (cons(copy_heap(car(x)), copy_heap(cdr(x))));
     return (x);
 }
