@@ -105,19 +105,17 @@ pass1(X) :-
     nl(user_output),
     abolish(pred_data/3),
     assert(pred_data(dummy,-1,-1)),
-    reconsult(X),
+    reconsult(X,compiler),
     pass1_analize.
 
 pass2(X) :-
     write(user_output,'phase pass2'),
     nl(user_output),
-    reconsult(X),
     pass1_analize.
 
 pass3(X) :-
     write(user_output,'phase pass3'),
     nl(user_output),
-    reconsult(X),
     pass1_analize.
 
 
@@ -302,7 +300,7 @@ gen_c_exec :-
 /*
 parts for gen_predicate
 C type declare.
-int b_foo(int arglist, int rest);
+static int b_foo(int arglist, int rest);
 */
 gen_type_declare(P) :-
 	write('static int b_'),
@@ -350,7 +348,7 @@ gen_var_declare1(S,E) :-
 
 /*
 generate predicate for not tail recursive
-int b_<name>(int arglist, int rest){
+static int b_<name>(int arglist, int rest){
 int varX,varY,...
 save2 = Jget_sp(th);
 save3 = Jget_ac(th);
@@ -715,6 +713,16 @@ gen_body1(((D1;D2),Xs),N) :-
     write(',th),'),
     gen_body1(Xs,N),
     write(',th)').
+
+gen_body1(((C1,C2),Xs),N) :-
+    write('Jwlist3(Jmakeope(","),'),
+	write('Jwlist3(Jmakeope(","),'),
+	gen_body1(C1,N),
+    write(','),
+    gen_body1(C2,N),
+    write(',th),'),
+    gen_body1(Xs,N),
+    write(',th)').
     
 gen_body1((X,Xs),N) :-
 	write('Jwlist3(Jmakeope(","),'),
@@ -736,14 +744,7 @@ gen_a_body((X;Xs)) :-
     write(','),
     gen_body1(Xs),
     write(',th)').
-% defined predicate will become compiled predicate
-gen_a_body(X) :-
-    n_defined_predicate(X),
-    functor(X,P,0),
-    not(n_dynamic_predicate(P)),
-    write('Jmakecomp("'),
-    write(P),
-    write('")').
+
 
 gen_a_body(X) :-
     n_defined_predicate(X),
@@ -753,6 +754,15 @@ gen_a_body(X) :-
     write(P),
     write('")').
 
+% defined predicate will become compiled predicate
+gen_a_body(X) :-
+    n_defined_predicate(X),
+    functor(X,P,0),
+    not(n_dynamic_predicate(P)),
+    write('Jmakecomp("'),
+    write(P),
+    write('")').
+
 % defined predicate will become compiled predicate
 gen_a_body(X) :-
     n_defined_predicate(X),
@@ -1175,12 +1185,28 @@ gen_a_argument(X) :-
     write('Jmakestr("'),
     write(X),
     write('")').
+gen_a_argument(X) :-
+    n_defined_predicate(X),
+    functor(X,Y,0),
+    n_dynamic_predicate(Y),
+    write('Jmakepred("'),
+    write(Y),
+    write('")').
 gen_a_argument(X) :-
     n_defined_predicate(X),
     functor(X,Y,0),
     write('Jmakecomp("'),
     write(Y),
     write('")').
+gen_a_argument(X) :-
+    n_defined_predicate(X),
+    X =.. [Y|Z],
+    n_dynamic_predicate(Y),
+    write('Jwcons(Jmakepred("'),
+    write(Y),
+    write('"),'),
+    gen_argument(Z),
+    write(',th)').
 gen_a_argument(X) :-
     n_defined_predicate(X),
     X =.. [Y|Z],
@@ -1262,6 +1288,14 @@ gen_a_argument(X) :-
     write('"),'),
     gen_argument(Z),
     write(',th)').
+gen_a_argument(X) :-
+    n_defined_userop(X),
+    X =.. [Y|Z],
+    write('Jwcons(Jmakecomp("'),
+    write(Y),
+    write('"),'),
+    gen_argument(Z),
+    write(',th)').
 gen_a_argument(X) :-
     n_property(X,userop),
     functor(X,Y,0),
@@ -1538,7 +1572,7 @@ gen_exec2(X) :-
     write('body = '),
     gen_body1(X,0),
     write(';'),nl,
-    write('Jexec_all(body,Jget_sp(th),th);'),!.
+    write('Jprove_all(body,Jget_sp(th),th);'),!.
 
 
 analize(P) :-
