@@ -17,7 +17,7 @@ address
  5,000,001 - 30,000,000  working area 
 30,000,001 - 32,000,000  variant area
 */
-#define VERSION     4.37
+#define VERSION     4.47
 #define CELLSIZE    30000000  // if raspberry PI set smaller size.
 #define HEAPSIZE     5000000
 #define FREESIZE         500
@@ -131,7 +131,7 @@ typedef struct {
 } cell;
 
 
-typedef enum toktype {LPAREN,RPAREN,LBRACKET,RBRACKET,VERTICAL,LCURL,RCURL,SHARP,
+typedef enum toktype {LPAREN,RPAREN,LBRACKET,RBRACKET,VERTICAL,LCURL,RCURL,
                       STRING,INTEGER,FLOATN,CHARCODE,ATOMOBJ,BUILTIN,FUNCTION,
                       COMPILED,OPERATOR,VARIABLE,ANOYMOUS,VARIANT,QUOTE,DOT,BACKQUOTE,DBLQUOTE,
                       COMMA,SEMICOLON,LONGNUM,BIGNUM,BINNUM,OCTNUM,HEXNUM,
@@ -186,7 +186,7 @@ enum { CONS_IDX, EQP_IDX, EQUALP_IDX, NUMEQP_IDX, SMALLERP_IDX, EQSMALLERP_IDX,
 
 enum { LIST3_IDX, ERRORCOMP_IDX, WLISTCONS_IDX,UNIFY_IDX, UNIFY_PAIR_IDX,
        UNIFY_INT_IDX, UNIFY_FLT_IDX, UNIFY_LONG_IDX, UNIFY_BIG_IDX, UNIFY_STR_IDX,
-       UNIFY_VAR_IDX, UNIFY_ATOM_IDX, EXEC_ALL_IDX, WCONS_IDX,
+       UNIFY_VAR_IDX, UNIFY_ATOM_IDX, EXEC_ALL_IDX, PROVE_ALL_IDX, WCONS_IDX,
        WLIST2_IDX, ADDTAIL_BODY_IDX, PLUS_IDX, MINUS_IDX, MULT_IDX, DIVIDE_IDX, REMAINDER_IDX,
        QUOTIENT_IDX,  MOD_IDX, EXPT_IDX, DIV_IDX,
        LEFTSHIFT_IDX, RIGHTSHIFT_IDX, LOGICALAND_IDX, LOGICALOR_IDX, ROUND_IDX,
@@ -557,6 +557,7 @@ extern int shutdown_flag;
 extern int active_thread;
 extern int dynamic_flag;
 extern int string_flag;
+extern int ifthenelse_false_flag;
 
 //------pointer----
 extern int hp; 
@@ -571,7 +572,7 @@ extern int cp[THREADSIZE];
 
 /* module */
 extern int module_name;
-extern int export_data[10][2];
+extern int export_data[256][2];
 extern int module_flag;
 extern int export_pt;
 
@@ -893,6 +894,7 @@ int b_halt(int arglist, int rest, int th);
 int b_has_cut(int arglist, int rest, int th);
 int b_heapdump(int arglist, int rest, int th);
 int b_inc(int arglist, int rest, int th);
+int b_initialization(int arglist, int rest, int th);
 int b_ifthen(int arglist, int rest, int th);
 int b_ifthenelse(int arglist, int rest, int th);
 int b_instance(int arglist, int rest, int th);
@@ -1019,7 +1021,6 @@ int b_var(int arglist, int rest, int th);
 int b_variable_convert(int arglist, int rest, int th);
 int b_unify_with_occurs_check(int arglist, int rest, int th);
 int b_write(int arglist, int rest, int th);
-int b_writeln(int arglist, int rest, int th);
 int b_writeq(int arglist, int rest, int th);
 int before_cut(int x);
 int before_cut1(int x, int y);
