@@ -309,6 +309,11 @@ int b_member(int arglist, int rest, int th)
 
 //---------extention--------
 
+/*
+	append([],X,X).
+	append([X|Ls],Ys,[X|Zs]):-
+        	append(Ls,Ys,Zs).
+*/
 int b_append(int arglist, int rest, int th)
 {
     int n, ind, arg1, arg2, arg3, x, ls, ys, zs, save1, save2, body;
@@ -331,14 +336,12 @@ int b_append(int arglist, int rest, int th)
 
 	save1 = wp[th];
 	if (unify_nil(arg1, th) == YES && unify(arg2, arg3, th) == YES) {
-	    if (prove(NIL, sp[th], rest, th) == YES)
+	    if (prove_all(rest, sp[th], th) == YES)
 		return (YES);
-	    else
-		return (NO);
 	}
 	wp[th] = save1;
 	unbind(save2, th);
-
+	
 	save1 = wp[th];
 	x = makevariant(th);
 	ls = makevariant(th);
@@ -346,10 +349,9 @@ int b_append(int arglist, int rest, int th)
 	zs = makevariant(th);
 	if (unify_pair(arg1, wlistcons(x, ls, th), th) == YES &&
 	    unify_var(arg2, ys, th) == YES
-	    && unify_pair(arg3, wlistcons(x, zs, th), th) == YES) {
+	    && unify(arg3, wlistcons(x, zs, th), th) == YES) {
 	    body = wlist4(makeatom("append", SYS), ls, ys, zs, th);
-	    if (prove(body, sp[th], rest, th) == YES)
-		return (YES);
+	    return (prove_all(addtail_body(rest,body,th), sp[th], th));
 	}
 	wp[th] = save1;
 	unbind(save2, th);
@@ -591,7 +593,7 @@ int b_bagofhelper(int arglist, int rest, int th)
 
     n = length(arglist);
     if (n == 1) {
-	arg1 = car(arglist);	//target var    
+	arg1 = car(arglist);	//target var  
 
 	nonfree = nonfree_list;
 	key = NIL;
@@ -599,7 +601,7 @@ int b_bagofhelper(int arglist, int rest, int th)
 	    key = cons(cons(car(nonfree), deref(car(nonfree), th)), key);
 	    nonfree = cdr(nonfree);
 	}
-	putinbag(key, deref(arg1, th));
+	putinbag(key, copy_heap(deref(arg1, th)));
 	return (NO);
     }
     return (NO);
@@ -1727,7 +1729,7 @@ int b_set_output(int arglist, int rest, int th)
 int b_use_module(int arglist, int rest, int th)
 {
     int n, ind, arg1;
-    char *home, str[STRSIZE];
+    char str[STRSIZE];
     FILE *fp;
 
     n = length(arglist);
@@ -1737,22 +1739,19 @@ int b_use_module(int arglist, int rest, int th)
 	if (!atomp(arg1))
 	    exception(NOT_ATOM, ind, arg1, th);
 
-	home = getenv("HOME");
-	strcpy(str, home);
-	strcat(str, "/nprolog/library/");
+	strcpy(str, "library/");
 	strcat(str, GET_NAME(arg1));
 	strcat(str, ".o");
-	fp = fopen(str, "r");
+	fp = fopen(prolog_file_name(str), "r");
 	if (fp != NULL) {
 	    fclose(fp);
 	    b_reconsult(list1(makeconst(str)), NIL, 0);
 	    return (prove_all(rest, sp[th], th));
 	}
-	strcpy(str, home);
-	strcat(str, "/nprolog/library/");
+	strcpy(str, "library/");
 	strcat(str, GET_NAME(arg1));
 	strcat(str, ".pl");
-	fp = fopen(str, "r");
+	fp = fopen(prolog_file_name(str), "r");
 	if (fp != NULL) {
 	    fclose(fp);
 	    b_reconsult(list1(makeconst(str)), NIL, 0);
@@ -1788,6 +1787,8 @@ int b_module(int arglist, int rest, int th)
 	    export_data[export_pt][1] = GET_INT(caddr(car(arg2)));
 	    arg2 = cdr(arg2);
 	    export_pt++;
+		if(export_pt >= 256)
+			exception(RESOURCE_ERR, NIL, ind, th);
 	}
 	return (prove_all(rest, sp[th], th));
     }
@@ -2457,3 +2458,19 @@ int b_string_atom(int arglist, int rest, int th)
     exception(ARITY_ERR, ind, arglist, th);
     return (NO);
 }
+
+
+int b_initialization(int arglist, int rest, int th)
+{
+    int n, arg1, ind;
+
+    n = length(arglist);
+    ind = makeind("initialization", n, th);
+    if (n == 1) {
+	arg1 = car(arglist);
+	execute_list = listcons(copy_heap(arg1),execute_list);
+	return(prove_all(rest,sp[th],th));
+	}
+    exception(ARITY_ERR, ind, arglist, th);
+    return (NO);
+}
\ No newline at end of file
